@@ -10,7 +10,7 @@ my $PORT = 12345;
 
 my $loop = IO::Async::Loop->new;
 
-my $listener = ChatListener->new();
+my $listener = ChatListener->new;
 
 $loop->add( $listener );
 
@@ -22,37 +22,34 @@ $listener->listen(
    on_listen_error  => sub { die "Cannot listen\n"; },
 );
 
-$loop->loop_forever;
+$loop->run;
 
 package ChatListener;
 use base qw( IO::Async::Listener );
 
 my @clients;
 
-sub on_accept
+sub on_stream
 {
    my $self = shift;
-   my ( $socket ) = @_;
+   my ( $stream ) = @_;
 
    # $socket is just an IO::Socket reference
+   my $socket = $stream->read_handle;
    my $peeraddr = $socket->peerhost . ":" . $socket->peerport;
 
    # Inform the others
    $_->write( "$peeraddr joins\n" ) for @clients;
 
-   my $clientstream = IO::Async::Stream->new(
-      handle => $socket,
-
+   $stream->configure(
       on_read => sub {
-         my ( $self, $buffref, $closed ) = @_;
+         my ( $self, $buffref, $eof ) = @_;
 
-         if( $$buffref =~ s/^(.*\n)// ) {
+         while( $$buffref =~ s/^(.*\n)// ) {
             # eat a line from the stream input
 
             # Reflect it to all but the stream who wrote it
             $_ == $self or $_->write( "$peeraddr: $1" ) for @clients;
-
-            return 1;
          }
 
          return 0;
@@ -67,6 +64,6 @@ sub on_accept
       },
    );
 
-   $loop->add( $clientstream );
-   push @clients, $clientstream;
+   $loop->add( $stream );
+   push @clients, $stream;
 }
