@@ -27,9 +27,8 @@ my $listener = IO::Async::Listener->new(
 
       $clientstream->write( "Your address is " . $peeraddr . "\n" );
 
-      $loop->resolve(
-         type => 'getnameinfo',
-         data => [ $socket->peername ],
+      $loop->resolver->getnameinfo(
+         addr => $socket->peername,
 
          on_resolved => sub {
             my ( $host, $service ) = @_;
@@ -54,4 +53,4 @@ $listener->listen(
    on_listen_error  => sub { die "Cannot listen\n"; },
 );
 
-$loop->loop_forever;
+$loop->run;
