@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Function;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.51';
+our $VERSION = '0.61';
 
 use base qw( IO::Async::Notifier );
 use IO::Async::Timer::Countdown;
@@ -113,6 +113,11 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 The body of the function to execute.
 
+=item model => "spawn" | "thread"
+
+Optional. Requests a specific C<IO::Async::Routine> model. If not supplied,
+leaves the default choice up to Routine.
+
 =item min_workers => INT
 
 =item max_workers => INT
@@ -156,7 +161,7 @@ sub _init
    $self->{min_workers} = 1;
    $self->{max_workers} = 8;
 
-   $self->{workers} = {};
+   $self->{workers} = {}; # {$id} => IaFunction:Worker
 
    $self->{pending_queue} = [];
 }
@@ -167,7 +172,7 @@ sub configure
    my %params = @_;
 
    my %worker_params;
-   foreach (qw( exit_on_die max_worker_calls )) {
+   foreach (qw( model exit_on_die max_worker_calls )) {
       $self->{$_} = $worker_params{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -189,17 +194,17 @@ sub configure
          $self->{idle_timer} = IO::Async::Timer::Countdown->new(
             delay => $timeout,
             on_expire => $self->_capture_weakself( sub {
-               my $self = shift;
+               my $self = shift or return;
                my $workers = $self->{workers};
 
                # Shut down atmost one idle worker, starting from the highest
-               # PID. Since we search from lowest to assign work, this tries
+               # ID. Since we search from lowest to assign work, this tries
                # to ensure we'll shut down the least useful ones first,
                # keeping more useful ones in memory (page/cache warmth, etc..)
-               foreach my $pid ( reverse sort keys %$workers ) {
-                  next if $workers->{$pid}{busy};
+               foreach my $id ( reverse sort keys %$workers ) {
+                  next if $workers->{$id}{busy};
 
-                  $workers->{$pid}->stop;
+                  $workers->{$id}->stop;
                   last;
                }
 
@@ -274,6 +279,7 @@ sub stop
 {
    my $self = shift;
 
+   $self->{stopping} = 1;
    foreach my $worker ( $self->_worker_objects ) {
       $worker->stop;
    }
@@ -332,6 +338,11 @@ circumstances given above. They will be called directly, without the leading
 
 =back
 
+=head2 $future = $function->call( %params )
+
+When returning a future, the C<on_result>, C<on_return> and C<on_error>
+continuations are optional.
+
 =cut
 
 sub call
@@ -345,15 +356,23 @@ sub call
    my $args = delete $params{args};
    ref $args eq "ARRAY" or croak "Expected 'args' to be an array";
 
-   my $on_result;
+   my $future = $self->loop->new_future;
+
    if( defined $params{on_result} ) {
-      my $inner_on_result = delete $params{on_result};
-      ref $inner_on_result or croak "Expected 'on_result' to be a reference";
-      $on_result = $self->_capture_weakself( sub {
-         my $self = shift;
-         $self->debug_printf( "CONT on_$_[0]" );
-         goto &$inner_on_result;
-      } );
+      my $on_result = delete $params{on_result};
+      ref $on_result or croak "Expected 'on_result' to be a reference";
+
+      $future->on_done( $self->_capture_weakself( sub {
+         my $self = shift or return;
+         $self->debug_printf( "CONT on_return" );
+         $on_result->( return => @_ );
+      } ) );
+      $future->on_fail( $self->_capture_weakself( sub {
+         my $self = shift or return;
+         my ( $err, @values ) = @_;
+         $self->debug_printf( "CONT on_error" );
+         $on_result->( error => @values );
+      } ) );
    }
    elsif( defined $params{on_return} and defined $params{on_error} ) {
       my $on_return = delete $params{on_return};
@@ -361,27 +380,33 @@ sub call
       my $on_error  = delete $params{on_error};
       ref $on_error or croak "Expected 'on_error' to be a reference";
 
-      $on_result = $self->_capture_weakself( sub {
-         my $self = shift;
-         my $result = shift;
-         $self->debug_printf( "CONT on_$result" );
-         $on_return->( @_ ) if $result eq "return";
-         $on_error->( @_ )  if $result eq "error";
-      } );
+      $future->on_done( $self->_capture_weakself( sub {
+         my $self = shift or return;
+         $self->debug_printf( "CONT on_return" );
+         $on_return->( @_ );
+      } ) );
+      $future->on_fail( $self->_capture_weakself( sub {
+         my $self = shift or return;
+         my ( $err, @values ) = @_;
+         $self->debug_printf( "CONT on_error" );
+         $on_error->( @values );
+      } ) );
    }
-   else {
-      croak "Expected either 'on_result' or 'on_return' and 'on_error' keys";
+   elsif( !defined wantarray ) {
+      croak "Expected either 'on_result' or 'on_return' and 'on_error' keys, or to return a Future";
    }
 
    my $worker = $self->_get_worker;
 
    if( !$worker ) {
       my $request = freeze( $args );
-      push @{ $self->{pending_queue} }, [ $request, $on_result ];
-      return;
+      push @{ $self->{pending_queue} }, [ $request, $future ];
+      return $future;
    }
 
-   $self->_call_worker( $worker, args => $args, $on_result );
+   $self->_call_worker( $worker, args => $args, $future );
+
+   return $future;
 }
 
 sub _worker_objects
@@ -431,13 +456,15 @@ sub _new_worker
    my $self = shift;
 
    my $worker = IO::Async::Function::Worker->new(
-      ( map { $_ => $self->{$_} } qw( code setup exit_on_die ) ),
+      ( map { $_ => $self->{$_} } qw( model code setup exit_on_die ) ),
       max_calls => $self->{max_worker_calls},
 
       on_finish => $self->_capture_weakself( sub {
          my $self = shift or return;
          my ( $worker ) = @_;
 
+         return if $self->{stopping};
+
          $self->_new_worker if $self->workers < $self->{min_workers};
 
          $self->_dispatch_pending;
@@ -446,7 +473,7 @@ sub _new_worker
 
    $self->add_child( $worker );
 
-   return $self->{workers}{$worker->pid} = $worker;
+   return $self->{workers}{$worker->id} = $worker;
 }
 
 sub _get_worker
@@ -467,9 +494,9 @@ sub _get_worker
 sub _call_worker
 {
    my $self = shift;
-   my ( $worker, $type, $args, $on_result ) = @_;
+   my ( $worker, $type, $args, $future ) = @_;
 
-   $worker->call( $type, $args, $on_result );
+   $worker->call( $type, $args, $future );
 
    if( $self->workers_idle == 0 ) {
       $self->{idle_timer}->stop if $self->{idle_timer};
@@ -547,14 +574,14 @@ sub stop
    $worker->{arg_channel}->close;
 
    if( my $function = $worker->parent ) {
-      delete $function->{workers}{$worker->pid};
+      delete $function->{workers}{$worker->id};
    }
 }
 
 sub call
 {
    my $worker = shift;
-   my ( $type, $args, $on_result ) = @_;
+   my ( $type, $args, $future ) = @_;
 
    if( $type eq "args" ) {
       $worker->{arg_channel}->send( $args );
@@ -576,10 +603,10 @@ sub call
          my $function = $worker->parent;
 
          if( $type eq "r" ) {
-            $on_result->( return => @values );
+            $future->done( @values );
          }
          elsif( $type eq "e" ) {
-            $on_result->( error => @values );
+            $future->fail( $values[0], @values );
             $worker->stop if $worker->{exit_on_die};
          }
          else {
@@ -597,7 +624,7 @@ sub call
 
          my $function = $worker->parent;
 
-         $on_result->( error => "closed" );
+         $future->fail( "closed", "closed" );
          $worker->stop;
 
          $function->_dispatch_pending if $function;
