@@ -1,23 +1,24 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop::Poll;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.51';
+our $VERSION = '0.61';
 use constant API_VERSION => '0.49';
 
 use base qw( IO::Async::Loop );
 
 use Carp;
 
-use IO::Poll qw( POLLIN POLLOUT POLLHUP POLLERR );
+use IO::Poll qw( POLLIN POLLOUT POLLPRI POLLHUP POLLERR );
 
-use POSIX qw( EINTR S_ISREG );
+use Errno qw( EINTR );
+use Fcntl qw( S_ISREG );
 
 # IO::Poll version 0.05 contain a bug whereby the ->remove method doesn't
 # properly clean up all the references to the handles. If the version we're
@@ -33,6 +34,12 @@ use constant _CAN_ON_HANGUP =>
 # poll() on most platforms claims that ISREG files are always read- and
 # write-ready, but not on MSWin32. We need to fake this
 use constant FAKE_ISREG_READY => IO::Async::OS->HAVE_FAKE_ISREG_READY;
+# poll() on most platforms indicates POLLOUT when connect() fails, but not on
+# MSWin32. Have to poll also for POLLPRI in that case
+use constant POLL_CONNECT_POLLPRI => IO::Async::OS->HAVE_POLL_CONNECT_POLLPRI;
+
+use constant _CAN_WATCHDOG => 1;
+use constant WATCHDOG_ENABLE => IO::Async::Loop->WATCHDOG_ENABLE;
 
 =head1 NAME
 
@@ -140,6 +147,8 @@ sub post_poll
 
    my $count = 0;
 
+   alarm( IO::Async::Loop->WATCHDOG_INTERVAL ) if WATCHDOG_ENABLE;
+
    foreach my $fd ( keys %$iowatches ) {
       my $watch = $iowatches->{$fd} or next;
 
@@ -154,7 +163,7 @@ sub post_poll
          $count++, $watch->[1]->() if defined $watch->[1];
       }
 
-      if( $events & (POLLOUT|POLLHUP|POLLERR) ) {
+      if( $events & (POLLOUT|POLLPRI|POLLHUP|POLLERR) ) {
          $count++, $watch->[2]->() if defined $watch->[2];
       }
 
@@ -167,6 +176,8 @@ sub post_poll
    # attempt to fire any waiting timeout events anyway
    $count += $self->_manage_queues;
 
+   alarm( 0 ) if WATCHDOG_ENABLE;
+
    return $count;
 }
 
@@ -245,11 +256,11 @@ sub watch_io
 
    my $mask = $curmask;
    $params{on_read_ready}  and $mask |= POLLIN;
-   $params{on_write_ready} and $mask |= POLLOUT;
+   $params{on_write_ready} and $mask |= POLLOUT | (POLL_CONNECT_POLLPRI ? POLLPRI : 0);
    $params{on_hangup}      and $mask |= POLLHUP;
 
-   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
-      $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+   if( FAKE_ISREG_READY and S_ISREG +(stat $handle)[2] ) {
+      $self->{fake_isreg}{$handle->fileno} = $mask;
    }
 
    $poll->mask( $handle, $mask ) if $mask != $curmask;
@@ -271,12 +282,12 @@ sub unwatch_io
 
    my $mask = $curmask;
    $params{on_read_ready}  and $mask &= ~POLLIN;
-   $params{on_write_ready} and $mask &= ~POLLOUT;
+   $params{on_write_ready} and $mask &= ~(POLLOUT | (POLL_CONNECT_POLLPRI ? POLLPRI : 0));
    $params{on_hangup}      and $mask &= ~POLLHUP;
 
-   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
+   if( FAKE_ISREG_READY and S_ISREG +(stat $handle)[2] ) {
       if( $mask ) {
-         $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+         $self->{fake_isreg}{$handle->fileno} = $mask;
       }
       else {
          delete $self->{fake_isreg}{$handle->fileno};
