@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::LoopTests;
 
@@ -23,11 +23,11 @@ use IO::Async::OS;
 
 use IO::File;
 use Fcntl qw( SEEK_SET );
-use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
+use POSIX qw( SIGTERM );
 use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.51';
+our $VERSION = '0.61';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -35,6 +35,7 @@ use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
 # The loop under test. We keep it in a single lexical here, so we can use
 # is_oneref tests in the individual test suite functions
 my $loop;
+END { undef $loop }
 
 =head1 NAME
 
@@ -157,7 +158,7 @@ Tests the Loop's ability to watch filehandles for IO readiness
 
 =cut
 
-use constant count_tests_io => 16;
+use constant count_tests_io => 17;
 sub run_tests_io
 {
    {
@@ -191,6 +192,25 @@ sub run_tests_io
       # Ready $S1 to clear the data
       $S1->getline; # ignore return
 
+      $loop->unwatch_io(
+         handle => $S1,
+         on_read_ready => 1,
+      );
+
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub { $readready = 1 },
+      );
+
+      $readready = 0;
+      $S2->syswrite( "more data\n" );
+
+      $loop->loop_once( 0.1 );
+
+      is( $readready, 1, '$readready after ->unwatch_io/->watch_io' );
+
+      $S1->getline; # ignore return
+
       $loop->watch_io(
          handle => $S1,
          on_write_ready => sub { $writeready = 1 },
@@ -375,7 +395,7 @@ Tests the Loop's ability to handle timer events
 
 =cut
 
-use constant count_tests_timer => 22;
+use constant count_tests_timer => 21;
 sub run_tests_timer
 {
    my $done = 0;
@@ -430,20 +450,22 @@ sub run_tests_timer
       $loop->loop_once while !$done;
    } 0, 0.1, 'loop_once while waiting for negative interval timer';
 
-   my $doneA;
-   my $doneB;
+   {
+      my $done;
 
-   $id = $loop->watch_time( after => 1 * AUT, code => sub {
-      $loop->unwatch_time( $id ); undef $id;
-      $doneA++;
-   });
+      my $id;
+      $id = $loop->watch_time( after => 1 * AUT, code => sub {
+         $loop->unwatch_time( $id ); undef $id;
+      });
 
-   $loop->watch_time( after => 1.1 * AUT, code => sub { $doneB++ } );
+      $loop->watch_time( after => 1.1 * AUT, code => sub {
+         $done++;
+      });
 
-   $loop->loop_once( 1 * AUT ) for 1 .. 3;
+      wait_for { $done };
 
-   is( $doneA, 1, 'Self-cancelling timer still fires' );
-   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
+      is( $done, 1, 'Other timers still fire after self-cancelling one' );
+   }
 
    # Legacy enqueue/requeue/cancel API
    $done = 0;
@@ -464,21 +486,25 @@ sub run_tests_timer
       }
    } 1.5, 2.5, 'loop_once(5) while waiting for timer';
 
-   # Check that short delays are achievable in one ->loop_once call
-   foreach my $delay ( 0.001, 0.01, 0.1 ) {
-      my $done;
-      my $count = 0;
-      my $start = time;
+   SKIP: {
+      skip "Unable to handle sub-second timers accurately", 3 unless $loop->_CAN_SUBSECOND_ACCURATELY;
 
-      $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+      # Check that short delays are achievable in one ->loop_once call
+      foreach my $delay ( 0.001, 0.01, 0.1 ) {
+         my $done;
+         my $count = 0;
+         my $start = time;
 
-      while( !$done ) {
-         $loop->loop_once( 1 );
-         $count++;
-         last if time - $start > 5; # bailout
+         $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+
+         while( !$done ) {
+            $loop->loop_once( 1 );
+            $count++;
+            last if time - $start > 5; # bailout
+         }
+
+         is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
       }
-
-      is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
    }
 
    $cancelled_fired = 0;
@@ -523,6 +549,11 @@ Tests the Loop's ability to watch POSIX signals
 use constant count_tests_signal => 14;
 sub run_tests_signal
 {
+   unless( IO::Async::OS->HAVE_SIGNALS ) {
+      SKIP: { skip "This OS does not have signals", 14; }
+      return;
+   }
+
    my $caught = 0;
 
    $loop->watch_signal( TERM => sub { $caught++ } );
@@ -662,7 +693,7 @@ sub run_in_child(&)
    die "Fell out of run_in_child!\n";
 }
 
-use constant count_tests_child => 8;
+use constant count_tests_child => 7;
 sub run_tests_child
 {
    my $kid = run_in_child {
@@ -679,29 +710,32 @@ sub run_tests_child
    undef $exitcode;
    wait_for { defined $exitcode };
 
-   ok( WIFEXITED($exitcode),      'WIFEXITED($exitcode) after child exit' );
-   is( WEXITSTATUS($exitcode), 3, 'WEXITSTATUS($exitcode) after child exit' );
+   ok( ($exitcode & 0x7f) == 0, 'WIFEXITED($exitcode) after child exit' );
+   is( ($exitcode >> 8), 3,     'WEXITSTATUS($exitcode) after child exit' );
 
-   # We require that SIGTERM perform its default action; i.e. terminate the
-   # process. Ensure this definitely happens, in case the test harness has it
-   # ignored or handled elsewhere.
-   local $SIG{TERM} = "DEFAULT";
+   SKIP: {
+      skip "This OS does not have signals", 1 unless IO::Async::OS->HAVE_SIGNALS;
 
-   $kid = run_in_child {
-      sleep( 10 );
-      # Just in case the parent died already and didn't kill us
-      exit( 0 );
-   };
+      # We require that SIGTERM perform its default action; i.e. terminate the
+      # process. Ensure this definitely happens, in case the test harness has it
+      # ignored or handled elsewhere.
+      local $SIG{TERM} = "DEFAULT";
 
-   $loop->watch_child( $kid => sub { ( undef, $exitcode ) = @_; } );
+      $kid = run_in_child {
+         sleep( 10 );
+         # Just in case the parent died already and didn't kill us
+         exit( 0 );
+      };
 
-   kill SIGTERM, $kid;
+      $loop->watch_child( $kid => sub { ( undef, $exitcode ) = @_; } );
 
-   undef $exitcode;
-   wait_for { defined $exitcode };
+      kill SIGTERM, $kid;
 
-   ok( WIFSIGNALED($exitcode),          'WIFSIGNALED($exitcode) after SIGTERM' );
-   is( WTERMSIG($exitcode),    SIGTERM, 'WTERMSIG($exitcode) after SIGTERM' );
+      undef $exitcode;
+      wait_for { defined $exitcode };
+
+      is( ($exitcode & 0x7f), SIGTERM, 'WTERMSIG($exitcode) after SIGTERM' );
+   }
 
    my %kids;
 
