@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Process;
 
@@ -9,17 +9,13 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.51';
+our $VERSION = '0.61';
 
 use Carp;
 
-use POSIX qw(
-   WIFEXITED WEXITSTATUS
-);
-
 use Socket qw( SOCK_STREAM );
 
-use Async::MergePoint 0.03;
+use Future;
 
 use IO::Async::OS;
 
@@ -111,7 +107,7 @@ sub _init
    $self->SUPER::_init( @_ );
 
    $self->{to_close}   = {};
-   $self->{mergepoint} = Async::MergePoint->new;
+   $self->{finish_futures} = [];
 }
 
 =head1 PARAMETERS
@@ -337,7 +333,7 @@ sub configure_fd
       $wants_read++;
    }
 
-   if( my $from = delete $args{from} ) {
+   if( defined( my $from = delete $args{from} ) ) {
       $opts->{from} = $from;
 
       $wants_write++;
@@ -381,7 +377,7 @@ sub _prepare_fds
    my $fd_handle = $self->{fd_handle};
    my $fd_opts   = $self->{fd_opts};
 
-   my $mergepoint = $self->{mergepoint};
+   my $finish_futures = $self->{finish_futures};
 
    my @setup;
 
@@ -441,16 +437,11 @@ sub _prepare_fds
          croak "Unsure what to do with fd_via==$via";
       }
 
-      unless( $write_only ) {
-         $mergepoint->needs( $key );
-         $handle->configure(
-            on_closed => sub {
-               $mergepoint->done( $key );
-            },
-         );
-      }
-
       $self->add_child( $handle );
+
+      unless( $write_only ) {
+         push @$finish_futures, $handle->new_close_future;
+      }
    }
 
    return @setup;
@@ -472,13 +463,10 @@ sub _add_to_loop
 
    push @setup, $self->_prepare_fds( $loop );
 
-   # Once we start the Process we'll close the MergePoint. Its on_finished
-   # coderef will strongly reference $self. So we need to break this cycle.
-   my $mergepoint = delete $self->{mergepoint};
-   
-   $mergepoint->needs( "exit" );
+   my $finish_futures = delete $self->{finish_futures};
 
    my ( $exitcode, $dollarbang, $dollarat );
+   push @$finish_futures, my $exit_future = $loop->new_future;
 
    $self->{pid} = $loop->spawn_child(
       code    => $self->{code},
@@ -488,7 +476,7 @@ sub _add_to_loop
 
       on_exit => sub {
          ( undef, $exitcode, $dollarbang, $dollarat ) = @_;
-         $mergepoint->done( "exit" );
+         $exit_future->done unless $exit_future->is_cancelled;
       },
    );
    $self->{running} = 1;
@@ -499,10 +487,9 @@ sub _add_to_loop
 
    my $is_code = defined $self->{code};
 
-   $mergepoint->close(
-      on_finished => $self->_capture_weakself( sub {
+   $self->{finish_future} = Future->needs_all( @$finish_futures )
+      ->on_done( $self->_capture_weakself( sub {
          my $self = shift or return;
-         my %items = @_;
 
          $self->{exitcode} = $exitcode;
          $self->{dollarbang} = $dollarbang;
@@ -524,6 +511,12 @@ sub _add_to_loop
    );
 }
 
+sub DESTROY
+{
+   my $self = shift;
+   $self->{finish_future}->cancel if $self->{finish_future};
+}
+
 sub notifier_name
 {
    my $self = shift;
@@ -590,7 +583,7 @@ C<exit(2)>.
 sub is_exited
 {
    my $self = shift;
-   return defined $self->{exitcode} ? WIFEXITED( $self->{exitcode} ) : undef;
+   return defined $self->{exitcode} ? ( $self->{exitcode} & 0x7f ) == 0 : undef;
 }
 
 =head2 $status = $process->exitstatus
@@ -603,7 +596,7 @@ passed to C<exit(2)>. Otherwise, returns C<undef>.
 sub exitstatus
 {
    my $self = shift;
-   return defined $self->{exitcode} ? WEXITSTATUS( $self->{exitcode} ) : undef;
+   return defined $self->{exitcode} ? ( $self->{exitcode} >> 8 ) : undef;
 }
 
 =head2 $exception = $process->exception
