@@ -1,18 +1,18 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Socket;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.61';
 
 use base qw( IO::Async::Handle );
 
-use POSIX qw( EAGAIN EWOULDBLOCK EINTR );
+use Errno qw( EAGAIN EWOULDBLOCK EINTR );
 
 use Carp;
 
@@ -26,7 +26,7 @@ filehandle
  use IO::Async::Socket;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  $loop->connect(
     host     => "some.host.here",
@@ -42,7 +42,7 @@ filehandle
              my ( $self, $dgram, $addr ) = @_;
 
              print "Received reply: $dgram\n",
-             $loop->loop_stop;
+             $loop->stop;
           },
           on_recv_error => sub {
              my ( $self, $errno ) = @_;
@@ -59,53 +59,52 @@ filehandle
     on_connect_error => sub { die "Cannot connect\n"; },
  );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
 This subclass of L<IO::Async::Handle> contains a socket filehandle. It
 provides a queue of outgoing data. It invokes the C<on_recv> handler when new
 data is received from the filehandle. Data may be sent to the filehandle by
-calling the C<send()> method.
+calling the C<send> method.
 
-It is primarily intended for C<SOCK_data> or C<SOCK_RAW> sockets; for
+It is primarily intended for C<SOCK_DGRAM> or C<SOCK_RAW> sockets; for
 C<SOCK_STREAM> sockets an instance of L<IO::Async::Stream> is probably more
 appropriate.
 
-This object may be used in one of two ways; as an instance with CODE
-references as callbacks, or as a base class with overridden methods.
+=head1 EVENTS
 
-=over 4
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-=item Subclassing
+=head2 on_recv $data, $addr
 
-If a subclass is built, then it can override the following methods to handle
-events:
-
- $self->on_recv( $data, $addr )
-
- $self->on_recv_error( $errno )
-
- $self->on_outgoing_empty()
-
- $self->on_send_error( $errno )
-
-=back
+Invoke on receipt of a packet, datagram, or stream segment.
 
 The C<on_recv> handler is invoked once for each packet, datagram, or stream
 segment that is received. It is passed the data itself, and the sender's
 address.
 
+=head2 on_recv_error $errno
+
+Optional. Invoked when the C<recv> method on the receiving handle fails.
+
+=head2 on_send_error $errno
+
+Optional. Invoked when the C<send> method on the sending handle fails.
+
 The C<on_recv_error> and C<on_send_error> handlers are passed the value of
 C<$!> at the time the error occured. (The C<$!> variable itself, by its
 nature, may have changed from the original error by the time this handler
 runs so it should always use the value passed in).
 
 If an error occurs when the corresponding error callback is not supplied, and
-there is not a subclass method for it, then the C<close()> method is
+there is not a subclass method for it, then the C<close> method is
 called instead.
 
-The C<on_outgoing_empty> handler is not passed any arguments.
+=head2 on_outgoing_empty
+
+Optional. Invoked when the sending data buffer becomes empty.
 
 =cut
 
@@ -138,31 +137,12 @@ Shortcut to specifying the same IO handle for both of the above.
 
 =item on_recv => CODE
 
-A CODE reference to invoke on receipt of a packet, datagram, or stream
-segment.
-
- $on_recv->( $self, $data, $addr )
-
 =item on_recv_error => CODE
 
-Optional. A CODE reference for when the C<recv()> method on the receiving
-handle fails.
-
- $on_recv_error->( $self, $errno )
-
 =item on_outgoing_empty => CODE
 
-Optional. A CODE reference for when the sending data buffer becomes empty.
-
- $on_outgoing_empty->( $self )
-
 =item on_send_error => CODE
 
-Optional. A CODE reference for when the C<send()> method on the sending handle
-fails.
-
- $on_send_error->( $self, $errno )
-
 =item autoflush => BOOL
 
 Optional. If true, the C<send> method will atempt to send data to the
@@ -171,7 +151,7 @@ filehandle is write-ready.
 
 =item recv_len => INT
 
-Optional. Sets the buffer size for C<recv()> calls. Defaults to 64 KiB.
+Optional. Sets the buffer size for C<recv> calls. Defaults to 64 KiB.
 
 =item recv_all => BOOL
 
@@ -215,8 +195,8 @@ sub configure
 
    $self->SUPER::configure( %params );
 
-   if( $self->get_loop and defined $self->read_handle ) {
-      $self->{on_recv} or $self->can( "on_recv" ) or
+   if( $self->loop and defined $self->read_handle ) {
+      $self->can_event( "on_recv" ) or
          croak 'Expected either an on_recv callback or to be able to ->on_recv';
    }
 }
@@ -226,7 +206,7 @@ sub _add_to_loop
    my $self = shift;
 
    if( defined $self->read_handle ) {
-      $self->{on_recv} or $self->can( "on_recv" ) or
+      $self->can_event( "on_recv" ) or
          croak 'Expected either an on_recv callback or to be able to ->on_recv';
    }
 
@@ -293,15 +273,8 @@ sub on_read_ready
 
          my $errno = $!;
 
-         if( defined $self->{on_recv_error} ) {
-            $self->{on_recv_error}->( $self, $errno );
-         }
-         elsif( $self->can( "on_recv_error" ) ) {
-            $self->on_recv_error( $errno );
-         }
-         else {
-            $self->close;
-         }
+         $self->maybe_invoke_event( on_recv_error => $errno )
+            or $self->close;
 
          return;
       }
@@ -311,10 +284,7 @@ sub on_read_ready
          return;
       }
 
-      my $on_recv = $self->{on_recv}
-                     || $self->can( "on_recv" );
-
-      $on_recv->( $self, $data, $addr );
+      $self->invoke_event( on_recv => $data, $addr );
 
       last unless $self->{recv_all};
    }
@@ -337,15 +307,8 @@ sub on_write_ready
 
          my $errno = $!;
 
-         if( defined $self->{on_send_error} ) {
-            $self->{on_send_error}->( $self, $errno );
-         }
-         elsif( $self->can( "on_send_error" ) ) {
-            $self->on_send_error( $errno );
-         }
-         else {
-            $self->close;
-         }
+         $self->maybe_invoke_event( on_send_error => $errno )
+            or $self->close;
 
          return;
       }
@@ -361,18 +324,10 @@ sub on_write_ready
    if( !$sendqueue or !@$sendqueue ) {
       $self->want_writeready( 0 );
 
-      my $on_outgoing_empty = $self->{on_outgoing_empty}
-                               || $self->can( "on_outgoing_empty" );
-
-      $on_outgoing_empty->( $self ) if $on_outgoing_empty;
+      $self->maybe_invoke_event( on_outgoing_empty => );
    }
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 SEE ALSO
 
 =over 4
@@ -386,3 +341,7 @@ L<IO::Handle> - Supply object methods for I/O handles
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
