@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.42';
+our $VERSION = '0.61';
 
 use Carp;
 
@@ -24,7 +24,7 @@ C<IO::Async::Timer::Absolute> - event callback at a fixed future time
  use POSIX qw( mktime );
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  my @time = gmtime;
 
@@ -33,13 +33,13 @@ C<IO::Async::Timer::Absolute> - event callback at a fixed future time
 
     on_expire => sub {
        print "It's midnight\n";
-       $loop->loop_stop;
+       $loop->stop;
     },
  );
 
  $loop->add( $timer );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
@@ -122,7 +122,7 @@ sub _make_cb
    my $self = shift;
 
    return $self->_capture_weakself( sub {
-      my ( $self ) = @_;
+      my $self = shift or return;
 
       undef $self->{id};
 
@@ -134,7 +134,7 @@ sub _make_enqueueargs
 {
    my $self = shift;
 
-   return time => $self->{time};
+   return at => $self->{time};
 }
 
 =head1 AUTHOR
